"use strict";

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

require("jest-styled-components");

var _theme = require("../../utils/theme");

var _Button = _interopRequireDefault(require("./Button"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('<Button />', function () {
  describe('snapshots', function () {
    it('renders default', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Button.default, {
        text: "Submit"
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
    it('renders disabled', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Button.default, {
        disabled: true,
        text: "Disabled"
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
    it('renders primary', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Button.default, {
        primary: true,
        text: "Primary"
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
    it('renders selected', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Button.default, {
        selected: true,
        text: "Selected"
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
    it('renders danger', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Button.default, {
        danger: true,
        text: "Danger"
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
  });
  it('runs the callback with the text prop', function () {
    var spy = jest.fn();
    var button = (0, _enzyme.shallow)(_react.default.createElement(_Button.default, {
      onClick: spy,
      text: "MyCustomText"
    }));
    button.simulate('click');
    expect(spy.mock.calls[0][1]).toBe('MyCustomText');
  });
  it('does not run the onClick callback when disabled', function () {
    var spy = jest.fn();
    var button = (0, _enzyme.shallow)(_react.default.createElement(_Button.default, {
      onClick: spy,
      text: "MyCustomText",
      disabled: true
    }));
    button.simulate('click');
    expect(spy.mock.calls.length).toEqual(0);
  });
  it('is okay when no onClick callback is provided', function () {
    var button = (0, _enzyme.shallow)(_react.default.createElement(_Button.default, {
      text: "MyCustomText",
      disabled: true
    })); // Should not throw

    button.simulate('click');
  });
  it('accepts a style prop', function () {
    var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Button.default, {
      style: {
        color: 'blue'
      }
    }))).toJSON();

    expect(tree).toMatchSnapshot();
  });
});