"use strict";

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

require("jest-styled-components");

var _theme = require("../../utils/theme");

var _Dialog = _interopRequireDefault(require("./Dialog"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('<Dialog />', function () {
  it('shows/hides when active/inactive', function () {
    var closeSpy = jest.fn();
    var dialog = (0, _enzyme.shallow)(_react.default.createElement(_Dialog.default, {
      active: true,
      onClose: closeSpy
    }));
    expect(dialog.get(0).props.active).toBe(true);
    dialog.find({
      text: ''
    }).simulate('click');
    expect(closeSpy).toBeCalled();
    dialog.setProps({
      active: false
    });
    expect(dialog.get(0).props.active).toBe(false);
  });
  it('runs the action-click handler', function () {
    var actionSpy = jest.fn();
    var dialog = (0, _enzyme.shallow)(_react.default.createElement(_Dialog.default, {
      active: true,
      actions: ['Submit', 'Cancel'],
      onActionClick: actionSpy
    }));
    dialog.find({
      text: 'Submit'
    }).simulate('click');
    expect(actionSpy).toBeCalledWith('Submit');
    dialog.find({
      text: 'Cancel'
    }).simulate('click');
    expect(actionSpy).toBeCalledWith('Cancel');
  });
  it('allows for React elements to be used for actions', function () {
    var newSpy = jest.fn();
    var dialog = (0, _enzyme.shallow)(_react.default.createElement(_Dialog.default, {
      active: true,
      actions: [_react.default.createElement("div", {
        className: "user-action",
        onClick: newSpy
      })]
    }));
    dialog.find('.user-action').simulate('click');
    expect(newSpy).toBeCalled();
  });
  it('should hide close icon when hideClose=true', function () {
    var wrapper = (0, _enzyme.shallow)(_react.default.createElement(_Dialog.default, {
      active: true
    }));
    expect(wrapper.find('i.fa.fa-close').length).toBe(1);
    wrapper.setProps({
      hideClose: true
    });
    expect(wrapper.find('i.fa.fa-close').length).toBe(0);
  });
  it('should only render child when active', function () {
    var heavyFn = jest.fn();

    var Child = function Child() {
      heavyFn();
      return _react.default.createElement("div", null, "Metal");
    };

    var wrapper = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Dialog.default, {
      active: false
    }, _react.default.createElement(Child, null))));
    expect(heavyFn).not.toHaveBeenCalled();
    wrapper.setProps({
      active: true
    });
    setTimeout(function () {
      expect(heavyFn).toHaveBeenCalledTimes(1);
    }, 0);
  });
  it('should call onClose when pressing esc', function () {
    // mock addEventListener so we can manually call the callback
    // https://github.com/airbnb/enzyme/issues/426
    var eventMap = {};
    document.addEventListener = jest.fn(function (event, cb) {
      eventMap[event] = cb;
    });
    var onCloseMock = jest.fn();
    var payload = {
      key: 'Escape'
    };
    var wrapper = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Dialog.default, {
      active: false,
      onClose: onCloseMock
    })));
    expect(onCloseMock).not.toHaveBeenCalled(); // call addEventListener callback

    eventMap.keydown(payload); // should not have been called as not active

    expect(onCloseMock).not.toHaveBeenCalled();
    wrapper.setProps({
      active: true
    }); // setProps is async

    setTimeout(function () {
      // call addEventListener callback
      eventMap.keydown(payload);
      expect(onCloseMock).toHaveBeenCalledTimes(1);
      expect(onCloseMock).toHaveBeenCalledWith(payload);
    }, 0);
  });
});