"use strict";

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

require("jest-styled-components");

var _theme = require("../../utils/theme");

var _Input = _interopRequireDefault(require("./Input"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('<Input />', function () {
  var props;
  var wrapper;
  var onChange;
  beforeEach(function () {
    onChange = jest.fn();
    props = {
      label: 'Hi',
      valid: false,
      message: 'Doh!',
      onChange: onChange
    };
    wrapper = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Input.default, props)));
  });
  it('should call onChange on input', function () {
    expect(onChange).not.toHaveBeenCalled();
    wrapper.find('input').simulate('change');
    expect(onChange).toHaveBeenCalledTimes(1);
  });
  it('should not render validation message when valid=true', function () {
    props = {
      valid: true,
      message: 'Derp!'
    };
    wrapper = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Input.default, props)), {
      lifecycleExperimental: true
    });
    expect(wrapper.find('Input__ValidationMessage')).toHaveStyleRule('visibility', 'hidden');
  });
  it('should render validation message when valid=false', function () {
    props = {
      valid: false,
      message: 'Derp!'
    };
    wrapper = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Input.default, props)));
    expect(wrapper.find('Input__ValidationMessage')).toHaveStyleRule('visibility', 'visible');
  });
  it('should remove validation message from the DOM when hideValidationMessage=true', function () {
    props = {
      hideValidationMessage: true
    };
    wrapper = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Input.default, props)));
    expect(wrapper.find('Input__ValidationMessage')).toHaveStyleRule('display', 'none');
  });
  it('should render correctly', function () {
    expect(wrapper).toMatchSnapshot();
  });
});