"use strict";

var _noop2 = _interopRequireDefault(require("lodash/noop"));

var _react = _interopRequireDefault(require("react"));

var _enzyme = require("enzyme");

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

require("jest-styled-components");

var _expect = require("expect");

var _theme = require("../../utils/theme");

var _Search = _interopRequireDefault(require("./Search"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('<Search />', function () {
  describe('snapshots', function () {
    it('renders empty', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Search.default, {
        query: "",
        pinnedTerms: [],
        onChange: _noop2.default,
        onFilterSelect: _noop2.default
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
    it('renders a search string', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Search.default, {
        query: "ui-server",
        pinnedTerms: [],
        onChange: _noop2.default,
        onFilterSelect: _noop2.default
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
    it('renders a group of terms', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Search.default, {
        query: "",
        pinnedTerms: ['deployments', 'default'],
        onChange: _noop2.default,
        onFilterSelect: _noop2.default
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
    it('renders filters', function () {
      var tree = _reactTestRenderer.default.create((0, _theme.withTheme)(_react.default.createElement(_Search.default, {
        query: "",
        pinnedTerms: [],
        onChange: _noop2.default,
        onFilterSelect: _noop2.default,
        filters: [{
          value: 'automated',
          label: 'Automated'
        }, {
          value: 'locked',
          label: 'Locked'
        }]
      }))).toJSON();

      expect(tree).toMatchSnapshot();
    });
  });
  it('returns search terms', function () {
    var spy = (0, _expect.createSpy)();
    var search = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Search.default, {
      query: "",
      pinnedTerms: ['a'],
      onChange: spy,
      onFilterSelect: _noop2.default
    })));
    var $input = search.find('input');
    $input.simulate('change', {
      target: {
        value: 'myterm'
      }
    });
    expect(spy.calls[0].arguments).toEqual(['myterm', ['a']]);
  });
  it('calls the onFilterSelect prop with selected filter value', function () {
    var spy = (0, _expect.createSpy)();
    var search = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Search.default, {
      query: "",
      pinnedTerms: [],
      onChange: _noop2.default,
      onFilterSelect: spy,
      filters: [{
        value: 'automated',
        label: 'Automated'
      }, {
        value: 'locked',
        label: 'Locked'
      }]
    }))); // Open the dialog pop up
    // TODO: make the dropdown do `display: none` instead of removing it from the dom?

    search.find('.dropdown-toggle').first().simulate('click'); // select a filter

    search.find('.dropdown-item').first().simulate('click');
    expect(spy.calls[0].arguments).toEqual(['automated']);
  });
  it('clears terms', function () {
    var spy = (0, _expect.createSpy)();
    var search = (0, _enzyme.mount)((0, _theme.withTheme)(_react.default.createElement(_Search.default, {
      query: "",
      pinnedTerms: ['a', 'b'],
      onChange: spy,
      onFilterSelect: _noop2.default
    }))); // Remove the 'a' search term

    search.find('.remove-term').first().simulate('click');
    expect(spy.calls[0].arguments).toEqual(['', ['b']]);
  });
});